//=============================================================================
// BattleSystem.cpp
//=============================================================================
#include "Stdinc.h"
#include "BattleSystem.h"
#include "DisplayManager.h"
#include "Manipulator.h"
#include "CommandManager.h"
#include "NumberAnimation.h"

#include "Character.h"
#include "Enemy.h"
#include "Field.h"
#include "TableAccess.h"
//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CBattleSystem::CBattleSystem(void)
	:m_dwSystemFrameCount( 0 ),m_nMode( MODE_INIT )
{
	// p[eB
	RtlZeroMemory( m_pPlayerChar, sizeof(m_pPlayerChar) );

	// Gp[eB
	RtlZeroMemory( m_pEnemy, sizeof(m_pEnemy) );

	for ( int iCnt = 0; iCnt < BGM_MAX; iCnt ++ )
	{
		m_nSndBGM[iCnt] = DX_FAILED;
	}

	// \}l[W
	m_pDisplayMan = new CDisplayManager();

	// Ǘ
	m_pManipulator = new CManipulator();

	// R}hǗ
	m_pCommandMan = new CCommandManager();

	// tB[h
	m_pField = new CField();
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CBattleSystem::~CBattleSystem(void)
{
	// TEh
	for ( int iCnt = 0; iCnt < BGM_MAX; iCnt ++ )
	{
		if ( m_nSndBGM[iCnt] != DX_FAILED )
		{
			DeleteSoundMem( m_nSndBGM[iCnt] );
		}
	}
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt] )
		{
			delete m_pPlayerChar[iCnt];
		}
	}
	for ( int iCnt = 0; iCnt < BATTLE_ENEMY_COUNT_MAX; iCnt ++ )
	{
		if ( m_pEnemy[iCnt] )
		{	delete m_pEnemy[iCnt];	}
	}
	delete m_pField;
	delete m_pManipulator;
	delete m_pCommandMan;
	delete m_pDisplayMan;
}
//-----------------------------------------------------------------------------
// GetInstance
//-----------------------------------------------------------------------------
CBattleSystem *CBattleSystem::GetInstance( void )
{
	static CBattleSystem m_Instance;
	return &m_Instance;
}
//-----------------------------------------------------------------------------
// Initialize
// 
//-----------------------------------------------------------------------------
BOOL CBattleSystem::Initialize( void )
{
	// [hݒ
	m_nMode = MODE_INIT;

	// t[JE^Zbg
	m_dwSystemFrameCount = 0;

	LoadSoundFiles();

	return TRUE;
}
//-----------------------------------------------------------------------------
// Debug_SetupPartyInfo
// fobOpp[eBݒ
//-----------------------------------------------------------------------------
void CBattleSystem::Debug_SetupPartyInfo( stPlayerPartyInfo *pPartyInfo, stPlayerCharacterInfo *pCharaInfo )
{
	CTableAccess::LoadDefaultPlayerPartyTable( pPartyInfo, pCharaInfo );
}
//-----------------------------------------------------------------------------
// SetupBattleParam
//-----------------------------------------------------------------------------
BOOL CBattleSystem::SetupBattleParam( const stBattleParam *pParam )
{
	BOOL bRet = FALSE;
	
	HaltBattle();

	// n`Awiݒ
	bRet &= m_pField->Initialize( pParam->Field.nID );

	// p[eBݒ
	SetupPlayerParty( &pParam->Party, pParam->Chara );

	// Gl~[ݒ
	SetupEnemy( &pParam->Enemy );

	return bRet;
}
//-----------------------------------------------------------------------------
// SetupPlayerParty
//-----------------------------------------------------------------------------
void CBattleSystem::SetupPlayerParty( const stPlayerPartyInfo *pPartyInfo, const stPlayerCharacterInfo *pCharaInfo )
{
	// p[eBS̏
	m_PartyParam = *pPartyInfo;

	// LN^ʏ
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt] )
		{	delete m_pPlayerChar[iCnt];	}

		m_pPlayerChar[iCnt] = new CCharacter();

		// p[^ݒ
		m_pPlayerChar[iCnt]->m_Param = pCharaInfo[iCnt];

		// 摜Ǎ
		m_pPlayerChar[iCnt]->LoadGraph();

		// \ʒuݒ
		int nPosX;
		if ( m_pPlayerChar[iCnt]->m_Param.nBtlPos == BATTLE_POS_FRONT )
			nPosX = FRONT_CHAR_POS_X + (iCnt * 5);
		else
			nPosX = BACK_CHAR_POS_X + (iCnt * 5);
		m_pPlayerChar[iCnt]->SetDispPos( nPosX ,  45 + (iCnt * 30) );	// ToDo: l

		
		// Aj[VJn
		m_pPlayerChar[iCnt]->StartAction( CCharacter::ANI_PTN_WALK_AT_START, NULL );
	}
}
//-----------------------------------------------------------------------------
// SetupEnemy
//-----------------------------------------------------------------------------
void CBattleSystem::SetupEnemy( const stEnemyPartyInfo *pEnemy )
{
	// 
	for ( int iCnt = 0; iCnt < BATTLE_ENEMY_COUNT_MAX; iCnt ++ )
	{
		if ( m_pEnemy[iCnt] )
		{	delete m_pEnemy[iCnt];	}

		m_pEnemy[iCnt] = NULL;
	}

	// EnemyPartyTable.csṽ[h
	CTableAccess::LoadEnemyPartyTable( pEnemy->nID, m_pEnemy, &m_nEnemyCount );

	// Aj[VJn
	for ( int iCnt = 0; iCnt < m_nEnemyCount; iCnt ++ )
	{
		m_pEnemy[iCnt]->StartAction( CEnemy::ANI_PTN_WALK_AT_START, NULL );
	}
}

//-----------------------------------------------------------------------------
// HaltBattle
//-----------------------------------------------------------------------------
void CBattleSystem::HaltBattle( void )
{
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt] )
		{
			m_pPlayerChar[iCnt]->Finalize();
		}
	}
	for ( int iCnt = 0; iCnt < BATTLE_ENEMY_COUNT_MAX; iCnt ++ )
	{
		if ( m_pEnemy[iCnt] )
		{
			m_pEnemy[iCnt]->Finalize();
		}
	}
	if ( m_pField )
	{
		m_pField->Finalize();
	}

	// TEh
	for ( int iCnt = 0; iCnt < BGM_MAX; iCnt ++ )
	{
		if ( m_nSndBGM[iCnt] != DX_FAILED )
		{
			StopSoundMem( m_nSndBGM[iCnt] );
		}
	}
}

//-----------------------------------------------------------------------------
// FrameMove
// XV
//-----------------------------------------------------------------------------
void CBattleSystem::FrameMove( void )
{
	// ANeBuł΃L[`FbN
	if ( GetActiveFlag() )
	{
		m_pManipulator->CheckKeyState();
	}

	// \}l[WXV
	m_pDisplayMan->FrameMove( m_pManipulator );

	// VXet[JEg
	m_dwSystemFrameCount ++;

	// [h
	if ( m_nMode == MODE_INIT )
	{	// 
		if ( m_pManipulator->IsAnyKeyPushed() )
		{
			// 퓬Jn
			StartBattle( NULL );
		}
		return;
	}

	if ( m_nMode == MODE_BATTLE )
	{	// 퓬

		// NĂ邩
		if ( !IsSomeoneActing() )
		{
			// R}hs
			CommandProc();
		}
		// GSŃ`FbN
		BOOL bAliving = FALSE;
		for ( int iCnt = 0; iCnt < m_nEnemyCount; iCnt ++ )
		{
			if ( m_pEnemy[iCnt]->IsAliving() )
			{
				bAliving = TRUE;
				break;
			}
		}
		if ( !bAliving )
		{	// GS
			m_nMode = MODE_WIN;
			m_pDisplayMan->ShowMessage( "" );
			StopSoundMem( m_nSndBGM[BGM_BATTLE] );
			PlaySoundMem( m_nSndBGM[BGM_WIN] , DX_PLAYTYPE_LOOP );
		}

		// SŃ`FbN
		bAliving = FALSE;
		for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
		{
			if ( m_pPlayerChar[iCnt]->IsAliving() )
			{
				bAliving = TRUE;
				break;
			}
		}
		if ( !bAliving )
		{	// S
			m_nMode = MODE_LOSE;
			m_pDisplayMan->ShowMessage( "sk" );
			StopSoundMem( m_nSndBGM[BGM_BATTLE] );
			PlaySoundMem( m_nSndBGM[BGM_LOSE] , DX_PLAYTYPE_LOOP );
		}
	}
	if ( m_nMode == MODE_WIN )
	{	// 

	}
	else if ( m_nMode == MODE_LOSE )
	{	// sk

	}
}
//-----------------------------------------------------------------------------
// IsSomeoneActing
//-----------------------------------------------------------------------------
BOOL CBattleSystem::IsSomeoneActing( void )
{
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt]->IsActing() )
			return TRUE;
	}
	for ( int iCnt = 0; iCnt < m_nEnemyCount; iCnt ++ )
	{
		if ( m_pEnemy[iCnt]->IsActing() )
			return TRUE;
	}

	return FALSE;
}
//-----------------------------------------------------------------------------
// GetActionablePlayer
//-----------------------------------------------------------------------------
int CBattleSystem::GetActionablePlayer( void )
{
	if ( m_lstAction.empty() )
		return -1;	// o^Ȃ

	int nIndex = m_lstAction.front();
	m_lstAction.pop_front();

	if ( !m_pPlayerChar[nIndex]->IsAliving() )
		return -1;	// ł

	return nIndex;
}
//-----------------------------------------------------------------------------
// CommandProc
//-----------------------------------------------------------------------------
void CBattleSystem::CommandProc( void )
{
	// R}h
	stCommandInfo CommandInfo;
	if ( m_pCommandMan->GetCommand( &CommandInfo ) == COMMAND_NONE )
	{
		return;
	}

	switch ( CommandInfo.nID )
	{
	// @p[eBR}h
	case COMMAND_PLAYER_ATTACK:		// 
		{
		// Param1Param2֕U

		if ( !m_pPlayerChar[CommandInfo.nParam1]->IsAliving() )
		{	// ł牽Ȃ
			break;
		}
		// _[WvZ
		int nDamage = CalcDamage( COMMAND_PLAYER_ATTACK, CommandInfo.nParam1, CommandInfo.nParam2 );

		// UAj[Vݒ
		stCommandInfo sysCommandInfo;
		sysCommandInfo.nID = COMMAND_SYSTEM_CHANGE_ENEMY_HP;
		sysCommandInfo.nParam1 = CommandInfo.nParam2;
		sysCommandInfo.nParam2 = nDamage;
		m_pPlayerChar[CommandInfo.nParam1]->StartAction( CCharacter::ANI_PTN_ATTACK, &sysCommandInfo );
		break;
		}
	case COMMAND_PLAYER_ITEM:		// ACe
		break;
	case COMMAND_PLAYER_DEFENCE:	// ڂ
		break;
	case COMMAND_PLAYER_CHANGE:		// `FW
		break;
	case COMMAND_PLAYER_RUN:		// 
		break;

	// @GR}h
	case COMMAND_ENEMY_ATTACK:		// G U
		// Param1Param2֕U
		if ( !m_pEnemy[CommandInfo.nParam1]->IsAliving() )
		{	// ł牽Ȃ
			break;
		}

		// 
		if ( CommandInfo.nParam2 == COMMAND_PARAM_RANDOM )
		{	// _ 
			CommandInfo.nParam2 = GetEnemyAttackTarget();
		}

		// _[WvZ
		int nDamage = CalcDamage( COMMAND_ENEMY_ATTACK, CommandInfo.nParam1, CommandInfo.nParam2 );

		// UAj[Vݒ
		m_pEnemy[CommandInfo.nParam1]->StartAction( CEnemy::ANI_PTN_ATTACK, NULL );

		// _[W炵
		stCommandInfo sysCommandInfo;
		sysCommandInfo.nID = COMMAND_SYSTEM_CHANGE_PLAYER_HP;
		sysCommandInfo.nParam1 = CommandInfo.nParam2;
		sysCommandInfo.nParam2 = nDamage;
		m_pEnemy[CommandInfo.nParam1]->StartAction( CEnemy::ANI_PTN_ATTACK, &sysCommandInfo );
		//printfDx("[GU] %d -> %d (ATK:%d)\n", CommandInfo.nParam1, CommandInfo.nParam2, nDamage );

		break;
	}
}
//-----------------------------------------------------------------------------
// ExpressCommandResult
// R}ȟʂ\
//-----------------------------------------------------------------------------
void CBattleSystem::ExpressCommandResult( int nSystemCmdID, int nParam1, int nParam2 )
{
	switch( nSystemCmdID )
	{
	case COMMAND_SYSTEM_CHANGE_ENEMY_HP:	// GHP
		// HP炵
		if ( !m_pEnemy[nParam1]->IsAliving() )
		{
			// ɎłIтȂ
			nParam1 = GetPlayerAttackTarget();
		}
		m_pEnemy[nParam1]->ChangeHitPoint( nParam2 );
		break;

	case COMMAND_SYSTEM_CHANGE_PLAYER_HP:	// v[HP
		// HP炵
		if ( !m_pPlayerChar[nParam1]->IsAliving() )
		{
			// ɎłIтȂ
			nParam1 = GetEnemyAttackTarget();
		}
		m_pPlayerChar[nParam1]->ChangeHitPoint( nParam2 );
		break;
	}
}
//-----------------------------------------------------------------------------
// GetPlayerAttackTarget
//-----------------------------------------------------------------------------
int CBattleSystem::GetPlayerAttackTarget( void )
{
	for ( int iCnt = 0; iCnt < m_nEnemyCount; iCnt ++ )
	{
		if ( m_pEnemy[iCnt]->IsAliving() )
			return iCnt;
	}

	return -1;
}
//-----------------------------------------------------------------------------
// GetEnemyAttackTarget
//-----------------------------------------------------------------------------
int CBattleSystem::GetEnemyAttackTarget( void )
{
	BOOL bLosed = TRUE;
	for ( int iCnt = 0; iCnt < BATTLE_CHAR_COUNT_MAX; iCnt ++ )
	{
		if ( m_pPlayerChar[iCnt]->IsAliving() )
		{
			bLosed = FALSE;
			break;
		}
	}

	if ( bLosed )	{	return -1;	}	// ɑSłĂ

	// ToDo : ǂ̃v[LIԂWbN
	int nIndex,iCnt = 0;
	for ( int iCnt = 0; iCnt < 3; iCnt ++ )
	{
		nIndex = GetRand(3);
		if ( !m_pPlayerChar[nIndex]->IsAliving() )
		{	// łIтȂ
			iCnt --;
			continue;
		}

		if ( m_pPlayerChar[nIndex]->m_Param.nBtlPos == BATTLE_POS_FRONT )
		{	// OqȂ
			return nIndex;
		}
	}
	// qI΂ꂽ
	return nIndex;
}

//-----------------------------------------------------------------------------
// CalcDamage
//-----------------------------------------------------------------------------
int CBattleSystem::CalcDamage( int nCommandID, int nParam1, int nParam2 )
{
	int nDamage = 0;
	if ( nCommandID == COMMAND_PLAYER_ATTACK )
	{	// v[̍U
		nDamage = m_pPlayerChar[nParam1]->m_Param.nAttack;
	}
	else if ( nCommandID == COMMAND_ENEMY_ATTACK )
	{	// G̍U
		nDamage = m_pEnemy[nParam1]->m_Param.nAttack;

	}

	return nDamage;
}

//-----------------------------------------------------------------------------
// StartBattle
//-----------------------------------------------------------------------------
BOOL CBattleSystem::StartBattle( const stBattleParam *pParam )
{
	BOOL bRet = FALSE;

	// 񏉊
	m_lstAction.clear();
	m_pCommandMan->RemoveAllCommand();

	if ( pParam == NULL )
	{	// ===== Kȏݒ	(Debug Only)

		stBattleParam BattleParam;
		ZeroMemory( &BattleParam, sizeof(BattleParam) );
		// n`Awiݒ
		BattleParam.Field.nID = 1;

		// p[eBݒ
		Debug_SetupPartyInfo( &BattleParam.Party, BattleParam.Chara );

		// GJEgGl~[ݒ
		BattleParam.Enemy.nID = 3;

		SetupBattleParam( &BattleParam );
	}
	else
	{
		SetupBattleParam( pParam );
	}

	// AACeʂł̃p[eB[C

	// ATBlݒ


	// \}l[W
	bRet &= m_pDisplayMan->Initialize( m_pPlayerChar, m_pEnemy, m_nEnemyCount, m_pField );
	
	// [hڍs
	m_nMode = MODE_BATTLE;

	// BGMĐ
	PlaySoundMem( m_nSndBGM[BGM_BATTLE] , DX_PLAYTYPE_LOOP );

	// eXgp
	//m_pDisplayMan->StartNumberAnimation( 200,100, 300, GetColor(255,255,255) );

	return bRet;
}
//-----------------------------------------------------------------------------
// LoadSoundFiles
//-----------------------------------------------------------------------------
BOOL CBattleSystem::LoadSoundFiles( void )
{
	m_nSndBGM[BGM_BATTLE]	= LoadSoundMem( BGM_FILE_NAME_01 );
	m_nSndBGM[BGM_WIN]		= LoadSoundMem( BGM_FILE_NAME_02 );
	m_nSndBGM[BGM_LOSE]		= LoadSoundMem( BGM_FILE_NAME_03 );

	return TRUE;
}
//-----------------------------------------------------------------------------
// Render
// `
//-----------------------------------------------------------------------------
void CBattleSystem::Render( void )
{
	m_pDisplayMan->Render();
}

//-----------------------------------------------------------------------------
// AddCommand
// R}hǉ
//-----------------------------------------------------------------------------
void CBattleSystem::AddCommand( int nCommandID, int nParam1, int nParam2 )
{
	m_pCommandMan->AddCommand( nCommandID, nParam1, nParam2 );
}

//-----------------------------------------------------------------------------
// AddCommand
// R}hǉ
//-----------------------------------------------------------------------------
void CBattleSystem::AddActionablePlayer( int nIndex )
{
	// łɓo^ĂȂΓo^
	std::list<int>::iterator it = m_lstAction.begin();
	while( it != m_lstAction.end() )
	{
		if ( nIndex == (*it) )
		{	// łɓo^Ă Ȃ
			return;
		}
		++it;
	}
	m_lstAction.push_back( nIndex );
}