//=============================================================================
// Character.cpp
//=============================================================================
#include "Stdinc.h"
#include "Character.h"
#include "BattleSystem.h"

const char *PLAYER_FILE_NAME[] = 
{
	PLAYER_FILE_NAME_01,PLAYER_FILE_NAME_02,
	PLAYER_FILE_NAME_03,PLAYER_FILE_NAME_04,
};


//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CCharacter::CCharacter(void)
	:m_pNumAnime( NULL ), m_nDispPicPtrn( STAND ), m_nAnimationPtrn(ANI_PTN_NONE), m_dwFrameCount( 0 )
{
	// lAj[V
	m_pNumAnime = new CNumberAnimation();

	ZeroMemory( &m_Param, sizeof(m_Param) );
	ZeroMemory( &m_Status, sizeof(m_Status) );
	ZeroMemory( &m_SysCommand, sizeof(m_SysCommand) );
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CCharacter::~CCharacter(void)
{
	// 摜̉
	Finalize();

	if ( m_pNumAnime )
	{
		delete m_pNumAnime;
	}
}
//-----------------------------------------------------------------------------
// LoadGraph
// 摜f[^Ǎ
//-----------------------------------------------------------------------------
BOOL CCharacter::LoadGraph( void )
{
	m_lpzFileName = PLAYER_FILE_NAME[m_Param.nID-1];
	// wiGǂݍ
	int iRet = LoadDivGraph( m_lpzFileName , CHAR_CHIP_COUNT, CHAR_CHIP_COUNT, 1, 27, 27, m_nHandle ) ;
	if ( iRet != DX_SUCCESS )
	{
		CUtility::tERROR("摜ǂݍݎs [%s]", m_lpzFileName );
		return FALSE;
	}
	return TRUE;
}
//-----------------------------------------------------------------------------
// Finalize
// I
//-----------------------------------------------------------------------------
void CCharacter::Finalize( void )
{
	for ( int iCnt = 0; iCnt < CHAR_CHIP_COUNT; iCnt ++ )
	{
		DeleteGraph( m_nHandle[iCnt] );
	}
}

//-----------------------------------------------------------------------------
// SetName
//-----------------------------------------------------------------------------
void CCharacter::SetName( const char *pName )
{
	strncpy_s( m_Param.szName, pName, _countof(m_Param.szName)-1 );
}
//-----------------------------------------------------------------------------
// StartAction
// Aj[Vp^[ݒ
//-----------------------------------------------------------------------------
void CCharacter::StartAction( int nPaternNo, stCommandInfo *pCommandInfo )
{
	m_nAnimationPtrn = nPaternNo;
	if ( pCommandInfo )
		m_SysCommand = *pCommandInfo;
	else
		RtlZeroMemory( &m_SysCommand, sizeof(m_SysCommand) );

	// Aj[VX^[gʒuݒ
	switch ( m_nAnimationPtrn )
	{
	case ANI_PTN_WALK_AT_START:	// Jns
		m_nDispPosX = m_nPosX + 50;
		break;
	default:
		// {ʒuɖ߂
		m_nDispPosX = m_nPosX;
		m_nDispPosY = m_nPosY;
	}

	// t[JE^Zbg
	m_dwFrameCount = 0;

}
//-----------------------------------------------------------------------------
// FrameMove
// XV
//-----------------------------------------------------------------------------
void CCharacter::FrameMove( void )
{
	switch ( m_nAnimationPtrn )
	{
	case ANI_PTN_NONE:			// Aj[V
		m_nDispPosX = m_nPosX;
		m_nDispPosY = m_nPosY;

		m_nDispPicPtrn = STAND;
		if ( m_Status.bDefence )
		{	// h䒆
			m_nDispPicPtrn = SITDOWN;
		}
		if ( m_Param.nHP <= 0 )
		{	// 퓬s\
			m_nDispPicPtrn = DEAD;
		}
		break;

	case ANI_PTN_WALK_AT_START:	// Jns
		if ( m_dwFrameCount % 6  < 3 )
		{
			m_nDispPicPtrn = WALK_R;
			m_nDispPosX -= 2;
		}
		else
		{
			m_nDispPosX -= 2;
			m_nDispPicPtrn = STAND;
		}

		if ( m_dwFrameCount == 25 )
		{
			StartAction( ANI_PTN_NONE, NULL );
		}
		break;

	case ANI_PTN_ATTACK:			// U

		if ( m_dwFrameCount <= 20 )
		{	// 20t[Oɕ
			if ( m_dwFrameCount % 6  < 3 )
			{
				m_nDispPicPtrn = WALK_R;
				m_nDispPosX -= 1;
			}
			else
			{
				m_nDispPosX -= 1;
				m_nDispPicPtrn = STAND;
			}
		}
		else if ( m_dwFrameCount <= 40 )
		{	// 20t[U[V
			if ( m_dwFrameCount % 4  < 2 )
			{
				m_nDispPicPtrn = ATACK_01;
			}
			else
			{
				m_nDispPicPtrn = ATACK_02;
			}
			if ( m_dwFrameCount == 40 )
			{	// Ut[Ō G nParam1HP nParam2炷
				CBattleSystem::GetInstance()->ExpressCommandResult( COMMAND_SYSTEM_CHANGE_ENEMY_HP, m_SysCommand.nParam1, m_SysCommand.nParam2 );
			}
		}
		else if ( m_dwFrameCount <= 60 )
		{	// 20t[ジ
			if ( m_dwFrameCount % 6  < 3 )
			{
				m_nDispPicPtrn = WALK_R;
				m_nDispPosX += 1;
			}
			else
			{
				m_nDispPosX += 1;
				m_nDispPicPtrn = STAND;
			}
		}
		else
		{
			StartAction( ANI_PTN_NONE, NULL );
		}
		break;
	}

	// lAj[V
	m_pNumAnime->FrameMove();

	// ATBl
	IncreaseATBValue( m_dwFrameCount );

	m_dwFrameCount ++;
}


//-----------------------------------------------------------------------------
// Render
// _O
//-----------------------------------------------------------------------------
void CCharacter::Render( void )
{
	// `
	DrawGraph( m_nDispPosX, m_nDispPosY, m_nHandle[m_nDispPicPtrn], TRUE );

	// HPAj[V
	m_pNumAnime->Render();
}
//-----------------------------------------------------------------------------
// GetNumAnimeDispPos
// Aj[V\ʒu擾
//-----------------------------------------------------------------------------
void CCharacter::GetNumAnimeDispPos( int *pX, int *pY, int iDispWidth )
{
	*pX = m_nPosX + ((27 - iDispWidth) / 2);
	*pY = m_nPosY + 27 - 12;
}
//-----------------------------------------------------------------------------
// IncreaseATBValue
// ATBl
//-----------------------------------------------------------------------------
void CCharacter::IncreaseATBValue( DWORD dwFrameCount )
{
	if ( dwFrameCount % 5 == 1 )
	{
		// ToDo : Xe[^Xɉ ATBlXV
		if ( m_Param.nATBValue < ATB_VALUE_MAX )
		{
			++ m_Param.nATBValue;
		}
		else
		{
			m_Param.nATBValue = ATB_VALUE_MAX;
		}
	}
}

//-----------------------------------------------------------------------------
// ChangeHitPoint
// HP
//-----------------------------------------------------------------------------
void CCharacter::ChangeHitPoint( int iDiff )
{
	// HPAj[Vݒ
	DWORD dwColor;
	if ( 0 <= iDiff )
	{	// 0Ɛ̒l
		dwColor = GetColor(255,255,255);
	}
	else
	{	// ̒l(HP)
		dwColor = GetColor(96,255,96);
	}

	int iX,iY;
	int iWidth = CNumberAnimation::GetDispWidth( iDiff );
	GetNumAnimeDispPos( &iX, &iY, iWidth );
	m_pNumAnime->StartAction( iX, iY, iDiff, dwColor );

	// HP炷
	m_Param.nHP -= iDiff;
	if ( m_Param.nHP < 0 )
		m_Param.nHP = 0;
}