//=============================================================================
// MenuWindow.cpp
//=============================================================================
#include "Stdinc.h"
#include "MenuWindow.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CMenuWindow::CMenuWindow(void)
	: m_nCurSel ( CURSOR_INDEX_NONE )
{
	// ACe
	RtlZeroMemory( m_Item, sizeof(m_Item) );
	for ( int iCnt = 0; iCnt < MENU_ITEM_COUNT_MAX; iCnt ++ )
	{
		m_Item[iCnt].nID = INVALID_ITEM_ID;
	}
}


//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CMenuWindow::~CMenuWindow(void)
{
}
//-----------------------------------------------------------------------------
// AddItem
//-----------------------------------------------------------------------------
void CMenuWindow::AddItem( int nIndex, int nID, const char *pText )
{
	// lRs[
	m_Item[nIndex].nID = nID;
	strcpy_s( m_Item[nIndex].szText, pText );

	if ( GetCurSel() == CURSOR_INDEX_NONE ) 
	{
		SetCurSel( nIndex );
	}
}
//-----------------------------------------------------------------------------
// Render
//-----------------------------------------------------------------------------
void CMenuWindow::Render( void )
{
	if ( !m_bShowFlag )
		return ;

	CFrameWindow::Render();

	// \
	for ( int iCnt = 0; iCnt < MENU_ITEM_COUNT_MAX; iCnt ++ )
	{
		if ( m_Item[iCnt].nID <= 0 )
		{
			continue;
		}

		int iPosX = m_nPosX + FRAME_TO_STRING_MARGINE_X;
		int iPosY = m_nPosY + FRAME_TO_STRING_MARGINE_Y + ( iCnt * (DISPLAY_MESSAGE_FONT_SIZE + 4) );

		DrawString( iPosX, iPosY, m_Item[iCnt].szText, STRING_COLOR );
	}
}
//-----------------------------------------------------------------------------
// RemoveItem
//-----------------------------------------------------------------------------
void CMenuWindow::RemoveItem( int nIndex )
{
	// 폜
	m_Item[nIndex].nID = INVALID_ITEM_ID;

	// IJ[\`FbN
	if ( m_Item[GetCurSel()].nID == INVALID_ITEM_ID )
	{
		SetCurSel(CURSOR_INDEX_NONE);
	}
}
//-----------------------------------------------------------------------------
// RemoveAllItems
//-----------------------------------------------------------------------------
void CMenuWindow::RemoveAllItems( void )
{
	for ( int iCnt = 0; iCnt < MENU_ITEM_COUNT_MAX; iCnt ++ )
	{
		RemoveItem( iCnt );
	}

	SetCurSel(CURSOR_INDEX_NONE);
}
//-----------------------------------------------------------------------------
// GetCursorDispPos
// J[\\ʒu擾
//-----------------------------------------------------------------------------
void CMenuWindow::GetCursorDispPos( int *pX, int *pY )
{
	if ( GetCurSel() == CURSOR_INDEX_NONE )
	{
		CUtility::tERROR2( "J[\G[" );
		SetCurSel(0);
	}
	*pX = m_nPosX - 8;
	*pY = m_nPosY + FRAME_TO_STRING_MARGINE_Y + ( GetCurSel() * (DISPLAY_MESSAGE_FONT_SIZE + 4) );
}

//-----------------------------------------------------------------------------
// NotifyKeyInput
//-----------------------------------------------------------------------------
BOOL CMenuWindow::NotifyKeyInput( int nKeyCode )
{
	int nCurSel = GetCurSel();
	int nNewSel;
	switch ( nKeyCode )
	{
	case KEY_INPUT_UP:		// 
		// J[\ړ
		nNewSel = nCurSel - 1;
		while ( 0 <= nNewSel )
		{
			if ( m_Item[nNewSel].nID != INVALID_ITEM_ID )
			{	break;	}	// break while
			nNewSel --;
		}
		if ( 0 <= nNewSel )
		{
			CUtility::PlaySound( CUtility::SND003 );
			SetCurSel( nNewSel );
		}
		break;

	case KEY_INPUT_DOWN:	// 
		// J[\ړ
		nNewSel = nCurSel +	1;
		while ( nNewSel < MENU_ITEM_COUNT_MAX  )
		{
			if ( m_Item[nNewSel].nID != INVALID_ITEM_ID )
			{
				break;	// break while
			}
			nNewSel ++;
		}
		if ( nNewSel < MENU_ITEM_COUNT_MAX )
		{
			CUtility::PlaySound( CUtility::SND003 );
			SetCurSel( nNewSel );
		}
		break;
	case KEY_INPUT_Z:	// 'Z' : LZL[
		CUtility::PlaySound( CUtility::SND002 );
		ShowWindow( FALSE );
		return FALSE;
	default:
		break;
	}

	return TRUE;
}
//-----------------------------------------------------------------------------
// NotifySubmit
//-----------------------------------------------------------------------------
int CMenuWindow::NotifySubmit( void )
{
	return m_Item[GetCurSel()].nID;
}

